////////////////////////////////////////////////////////////////
//                                               //
////////////////////////////////////////////////////////////////

unit UMyClass;

interface

uses ExtCtrls, Graphics, SysUtils;

type
     TMyGraph = class      //   
     private
       FImg   :TImage;                       //    
       FXmin  :double;                       //   X
       FXmax  :double;                       //   X
       FYmin  :double;                       //   Y
       FYmax  :double;                       //   Y
       function F(X:double):double; virtual; // 
       function Xe(X:double):longint;        //     X
       function Ye(Y:double):longint;        //     Y
       procedure Clear;                      //  
       procedure Grids;                      // 
       procedure Show;                       //  
     public
       constructor Create(Img:TImage);       //  
       destructor Destroy; override;         //  
       procedure Run;                        //  
       property Xmin:double read FXmin write FXmin;  //   X
       property Xmax:double read FXmax write FXmax;  //   X
       property Ymin:double read FYmin write FYmin;  //   Y
       property Ymax:double read FYmax write FYmax;  //   Y
     end;

     TMySin = class(TMyGraph)  //   
     private
       function F(X:double):double; override; // 
     public
     end;

     TMyCos = class(TMyGraph)  //   
     private
       function F(X:double):double; override; // 
     public
     end;


var  GR :TMyGraph;   //  

implementation


////////////////////////////////////////////////////////////////
//                   TMyGraph                   //
////////////////////////////////////////////////////////////////

function TMyGraph.F(X:double):double;  //  ()
begin
     result:=0;
end;

function TMyGraph.Xe(X:double):longint;  //     X
begin
     if FImg=nil then result:=Trunc(X)
                 else result:=Trunc((FImg.ClientWidth-1)*(X-FXmin)/(FXmax-FXmin));
end;

function TMyGraph.Ye(Y:double):longint;  //     Y
begin
     if FImg=nil then result:=Trunc(Y)
                 else result:=FImg.ClientHeight-Trunc((FImg.ClientHeight-1)*(Y-FYmin)/(FYmax-FYmin));
end;

procedure TMyGraph.Clear;  //  
begin
     if FImg=nil then Exit;
     FImg.Canvas.Brush.Color:=clWhite;
     FImg.Canvas.FillRect(FImg.ClientRect);
end;

procedure TMyGraph.Grids;  // 
const  H = 0.1;    //  
var  X,Y :double;    //  X,Y
     D   :longint;   //  
begin
     if FImg=nil then Exit;
     // 
     FImg.Canvas.Brush.Color:=clBlack;
     FImg.Canvas.FrameRect(FImg.ClientRect);
     //  
     FImg.Canvas.Pen.Color:=clGray;
     FImg.Canvas.MoveTo(Xe(FXmin),Ye(0));
     FImg.Canvas.LineTo(Xe(FXmax),Ye(0));
     //    
     FImg.Canvas.Font.Color:=clBlack;
     FImg.Canvas.Brush.Color:=clWhite;
     X:=Trunc(FXmin/H)*H;
     While X<=FXmax do begin
           if Abs(X-Round(X))<(H/2) then begin
              D:=5;
              FImg.Canvas.TextOut(Xe(X)-3,Ye(0)+D+3,FloatToStr(Round(X)));
           end else D:=2;
           FImg.Canvas.MoveTo(Xe(X),Ye(0)-D);
           FImg.Canvas.LineTo(Xe(X),Ye(0)+D);
           X:=X+H;
     end;
     //  
     FImg.Canvas.MoveTo(Xe(0),Ye(FYmax));
     FImg.Canvas.LineTo(Xe(0),Ye(FYmin));
     //    
     Y:=Trunc(FYmin/H)*H;
     While Y<=FYmax do begin
           if Abs(Y-Round(Y))<(H/2) then begin
              D:=5;
              if Round(Y)<>0 then
                 FImg.Canvas.TextOut(Xe(0)+D+3,Ye(Y)-5,FloatToStr(Round(Y)));
           end else D:=2;
           FImg.Canvas.MoveTo(Xe(0)-D,Ye(Y));
           FImg.Canvas.LineTo(Xe(0)+D,Ye(Y));
           Y:=Y+H;
     end;
end;

procedure TMyGraph.Show;  //  
const  H = 0.1;
var  X :double;
begin
     if FImg=nil then Exit;
     FImg.Canvas.Pen.Color:=clBlue;
     X:=FXmin;  FImg.Canvas.MoveTo(Xe(X),Ye(F(X)));
     While X<=FXmax do begin
           FImg.Canvas.LineTo(Xe(X),Ye(F(X)));
           X:=X+H;
     end;
end;

constructor TMyGraph.Create(Img:TImage);  //  
begin
     FImg:=Img;      //    
     FXmin  :=-10;   //   X
     FXmax  :=10;    //   X
     FYmin  :=-2;    //   Y
     FYmax  :=2;     //   Y
end;

destructor TMyGraph.Destroy;  //  
begin
     FImg:=nil;
end;

procedure TMyGraph.Run;  //  
begin
     Clear;   //  
     Grids;   //  
     Show;    //  
end;


////////////////////////////////////////////////////////////////
//                   TMySin                     //
////////////////////////////////////////////////////////////////

function TMySin.F(X:double):double;  // 
begin
     result:=sin(X);
end;


////////////////////////////////////////////////////////////////
//                   TMyCos                     //
////////////////////////////////////////////////////////////////

function TMyCos.F(X:double):double;  // 
begin
     result:=cos(X);
end;


initialization
   GR:=TMyGraph.Create(nil);
finalization
   GR.Free;
end.
